package org.maachang.mailslot ;

/**
 * メールスロットオブジェクト.
 *  
 * @version 2008/03/10
 * @author  masahito suzuki
 * @since  MailSlot 1.00
 */
public class MailSlot {
    private static final MailSlot SNGL = new MailSlot() ;
    private NativeMailSlot ntv = null ;
    
    private MailSlot() {
        
    }
    
    /**
     * オブジェクトを取得.
     * <BR>
     * @return MailSlot オブジェクトが返されます.
     */
    public static final MailSlot getInstance() {
        return SNGL ;
    }
    
    private void createNative()
        throws Exception {
        if( ntv == null ) {
            ntv = new NativeMailSlot() ;
        }
    }
    
    /**
     * 指定名のメールスロットをオープン.
     * <BR>
     * @param name 新しいメールスロット名をオープンします.
     * @exception Exception 例外.
     */
    public void create( String name ) throws Exception {
        createNative() ;
        ntv.closeMailSlot() ;
        ntv.openMailSlot( name ) ;
    }
    
    /**
     * メールスロットをクローズ.
     * <BR>
     * @exception Exception 例外.
     */
    public void close() throws Exception {
        createNative() ;
        ntv.closeMailSlot() ;
    }
    
    /**
     * メールスロットから情報を受信.
     * <BR>
     * @return byte[] メールスロットから情報を受信します.
     * @exception Exception 例外.
     */
    public byte[] receive() throws Exception {
        createNative() ;
        return ntv.readMailSlot() ;
    }
    
    /**
     * プロセス間通信でメールスロットに情報を送信.
     * <BR>
     * @param name 対象のメールスロット名を設定します.
     * @param data 対象のデータを設定します.
     * @exception Exception 例外.
     */
    public void send( String name,byte[] data )
        throws Exception {
        createNative() ;
        ntv.writeMailSlot( null,name,data ) ;
    }
    
    /**
     * メールスロットに情報を送信.
     * <BR>
     * @paream compName 送信対象のコンピュータ名を設定します.<BR>
     *                  [null]を設定した場合、プロセス間での通信になります.
     * @param name 対象のメールスロット名を設定します.
     * @param data 対象のデータを設定します.
     * @exception Exception 例外.
     */
    public void send( String compName,String name,byte[] data )
        throws Exception {
        createNative() ;
        ntv.writeMailSlot( compName,name,data ) ;
    }
}
